b3p clean blade_test.yml

tree temp_blade_portable

b3p build blade_test.yml

tree temp_blade_portable

b3p 2d blade_test.yml

tree temp_blade_portable

b3p ccx blade_test.yml

tree temp_blade_portable
